/*
 * Copyright (c) 1988 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/* Data structures for the display in the osimon program */

/* external record format */

typedef struct
{
    int  id1;                   /* primary key */
    int  id2;                   /* secondary key */
} COMPID;


typedef struct
{
    COMPID  f_compid;           /* component id */
    int     f_time;             /* timestamp */
    int     f_len;              /* length of string */
    char   *f_text;             /* text of record */
} FREC;

#define FRECSZ (sizeof(FREC))


#ifndef NULLIFP
#define NULLIFP ((int (*)()) 0)
#endif


/*driving table format*/

typedef struct
{
    int   (*df_getrec) ();      /* routine to get a rec from external source*/
    int   (*df_open) ();        /* routine to "open" the source */
    int   (*df_close) ();       /* routine to "close" the source */
    int   (*df_order) ();       /* routine to order the records */
    char  **df_hdr1a;           /* header 1, line a fields */
    char  **df_hdr1b;           /* header 1, line b fields */
    char  **df_hdr2a;           /* header 2, line a fields */
    char  **df_hdr2b;           /* header 2, line b fields */
    char   *df_tbl1fmt;         /* first tbl format */
    char   *df_tbl2fmt;         /* second tbl format */
    char  **df_msg;             /* text messages */
    char   *df_title;           /* title */
} DISFL;



/*control codes*/

#define UPONE   '\013'
#define DOWNONE '\012'
#define UPPAGE  '\002'
#define DOWNPAGE '\006'
#define CLEAR   '\014'
#define HOME    '\035'
#define ESCAPE  '\033'


#define LNLEN           80      /* maximum line length */

#define GAP             0       /* gap between displayed lines */
#define TTLNS           1       /* no. of lines in title */
#define HDRLNS          4       /* lines in headers mixed with recs
				 * (manipulated as recs) */

#define BLANK           2
#define HEADER          1
#define UNDLN           0

#define HDR1           -100
#define HDR2           -50

#define HDR1_BLANK     -16
#define HDR1A          -15
#define HDR1B          -14
#define HDR1_UNDLN     -13

#define GRP1           -10

#define HDR2_BLANK     -6
#define HDR2A          -5
#define HDR2B          -4
#define HDR2_UNDLN     -3

#define GRP2            0

#define NEW_GRP1        1
#define NEW_GRP2        2
