/*
 * Copyright (c) 1988 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/* Header file for Common Management Information Service providers  */

/*
 * By George Pavlou, April 1990
 */

#include "mpep.h"


struct MSAPassocinfo
{
    int        mai_protvrsn;	/* protocol version */
    int        mai_funits;	/* functional units */
    External * mai_access;	/* access control */
    External * mai_info;	/* user info */
};

struct MSAPconnect
{
    int    mc_sd;		/* connection descriptor */
    int    mc_result;
    OID    mc_context;
    AEInfo mc_resptitle;
    struct MSAPassocinfo mc_info;
};

struct MSAPstart
{
    int    ms_sd;
    AEInfo ms_calling;
    AEInfo ms_called;
    struct PSAPstart ms_start;
    OID    ms_context;
    struct MSAPassocinfo ms_info;
};

struct MSAPterm
{
    int mt_reason;
};
    
struct MSAPpreject
{
    int  mr_reason;
    int  mr_id;
#define MR_SIZE	64
    char mr_data[MR_SIZE];
};

struct MSAPabort
{
    int  ma_source;
#define MA_USER		0
#define MA_PROVIDER	1
#define MA_LOCAL	2

    int  ma_reason;
#define MA_NOVRSN       30	/* no common CMIP version */
#define MA_BADPCTX      31	/* bad presentation context */
#define MA_BADARGS      32	/* bad argument */
#define MA_BADINFO	33	/* bad assoc user info */
#define MA_ROS          34	/* can't set ROS service */
#define MA_NOTREL       35	/* peer refused to release */

    External * ma_info;
#define MA_SIZE	64
    char ma_data[MA_SIZE];
};

struct MSAPerror
{
    int id;
    struct CMISerrparam info;
};

struct MSAPeventrepv
{
    int id;
    struct event_rep_arg args;
};

struct MSAPeventrepr
{
    int id;
    struct event_rep_res result;
};

struct MSAPlinkedreplyv
{
    int id;
    struct linked_reply_arg args;
};

struct MSAPgetv
{
    int id;
    struct get_arg args;
};

struct MSAPgetr
{
    int id;
    struct get_res result;
};

struct MSAPsetv
{
    int id;
    struct set_arg args;
};

struct MSAPsetr
{
    int id;
    struct set_res result;
};

struct MSAPactionv
{
    int id;
    struct action_arg args;
};

struct MSAPactionr
{
    int id;
    struct action_res result;
};

struct MSAPcreatev
{
    int id;
    struct create_arg args;
};

struct MSAPcreater
{
    int id;
    struct create_res result;
};

struct MSAPdeletev
{
    int id;
    struct delete_arg args;
};

struct MSAPdeleter
{
    int id;
    struct delete_res result;
};

struct MSAPindication
{
    int mi_type;
#define MI_UNKNOWN	0
#define MI_TERM		1
#define MI_ABORT	2
#define MI_PREJECT	3
#define MI_ERROR	4
#define MI_EVENT_REP	5
#define MI_EVENT_REPC	6
#define MI_EVENT_REP_RES 7
#define MI_LINKED_REPLY 8
#define MI_GET		9
#define MI_GET_RES	10
#define MI_SET		11
#define MI_SETC		12
#define MI_SET_RES	13
#define MI_ACTION	14
#define MI_ACTIONC	15
#define MI_ACTION_RES	16
#define MI_CREATE	17
#define MI_CREATE_RES	18
#define MI_DELETE	19
#define MI_DELETE_RES	20

    CMISerrors mi_errtype;

    union {
	struct MSAPterm mi_un_term;
	struct MSAPabort mi_un_abort;
	struct MSAPpreject mi_un_prej;
	struct MSAPerror mi_un_error;
	struct MSAPeventrepv mi_un_eventreportv;
	struct MSAPeventrepr mi_un_eventreportr;
	struct MSAPlinkedreplyv mi_un_linkedreplyv;
	struct MSAPgetv mi_un_getv;
	struct MSAPgetr mi_un_getr;
	struct MSAPsetv mi_un_setv;
	struct MSAPsetr mi_un_setr;
	struct MSAPactionv mi_un_actionv;
	struct MSAPactionr mi_un_actionr;
	struct MSAPcreatev mi_un_createv;
	struct MSAPcreater mi_un_creater;
	struct MSAPdeletev mi_un_deletev;
	struct MSAPdeleter mi_un_deleter;
    } mi_un;
#define mi_term		mi_un.mi_un_term
#define mi_preject	mi_un.mi_un_prej
#define mi_abort	mi_un.mi_un_abort
#define mi_error	mi_un.mi_un_error
#define mi_eventrepv	mi_un.mi_un_eventreportv
#define mi_eventrepr	mi_un.mi_un_eventreportr
#define mi_linkedreplyv mi_un.mi_un_linkedreplyv
#define mi_getv		mi_un.mi_un_getv
#define mi_getr		mi_un.mi_un_getr
#define mi_setv		mi_un.mi_un_setv
#define mi_setr		mi_un.mi_un_setr
#define mi_actionv	mi_un.mi_un_actionv
#define mi_actionr	mi_un.mi_un_actionr
#define mi_createv	mi_un.mi_un_createv
#define mi_creater	mi_un.mi_un_creater
#define mi_deletev	mi_un.mi_un_deletev
#define mi_deleter	mi_un.mi_un_deleter
};


#define M_ACCEPT ACS_ACCEPT

#define M_EVENT_REP	0
#define M_EVENT_REPC	1
#define M_LINKED_REPLY	2
#define M_GET		3
#define M_SET		4
#define M_SETC		5
#define M_ACTION	6
#define M_ACTIONC	7
#define M_CREATE	8
#define M_DELETE	9


#define M_EventRep(a, b, c, d, e, f, g, h) \
	M_EventRepAux(a, M_EVENT_REP, b, c, d, e, f, g, h)
#define M_EventRepC(a, b, c, d, e, f, g, h) \
	M_EventRepAux(a, M_EVENT_REPC, b, c, d, e, f, g, h)
#define M_Set(a, b, c, d, e, f, g, h, i, j, k) \
	M_SetAux(a, M_SET, b, c, d, e, f, g, h, i, j, k)
#define M_SetC(a, b, c, d, e, f, g, h, i, j, k) \
	M_SetAux(a, M_SETC, b, c, d, e, f, g, h, i, j, k)
#define M_Action(a, b, c, d, e, f, g, h, i, j, k) \
	M_ActionAux(a, M_ACTION, b, c, d, e, f, g, h, i, j, k)
#define M_ActionC(a, b, c, d, e, f, g, h, i, j, k) \
	M_ActionAux(a, M_ACTIONC, b, c, d, e, f, g, h, i, j, k)

int	M_EventRepAux ();	/* M-EVENT-REPORT.REQUEST (conf./non-conf.) */
int	M_EventReportRes ();	/* M-EVENT-REPORT.RESPONSE */
int	M_Get ();		/* M-GET.REQUEST */
int	M_GetRes ();		/* M-GET.RESPONSE */
int	M_SetAux ();		/* M-SET.REQUEST (confirmed/non-conf.) */
int	M_SetRes ();		/* M-SET.RESPONSE */
int	M_ActionAux ();		/* M-ACTION.REQUEST (confirmed/non-conf.) */
int	M_ActionRes ();		/* M-ACTION.RESPONSE */
int	M_Create ();		/* M-CREATE.REQUEST */
int	M_CreateRes ();		/* M-CREATE.RESPONSE */
int	M_Delete ();		/* M-DELETE.REQUEST */
int	M_DeleteRes ();		/* M-DELETE.RESPONSE */
int	M_WaitReq ();		/* M-WAIT.REQUEST (pseudo) */

int	M_InitialiseReq ();	/* CMIS A-ASSOCIATE.REQUEST */
int	MInit ();		/* CMIS A-ASSOCIATE.INDICATION */
int	M_InitialiseResp ();	/* CMIS A-ASSOCIATE.RESPONSE */
int	M_TerminateReq ();	/* CMIS A-RELEASE.REQUEST */
int	M_TerminateResp ();	/* CMIS A-RELEASE.RESPONSE */
int	M_AbortReq ();		/* CMIS A-ABORT.REQUEST */

int	mifree ();		/* free MSAPindication struct */

OID	oid_copy ();		/* copies to an existing OIDentifier struct */
