/*
 * Copyright (c) 1988 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/* objectS.h - simplified definitions for specific Managed Objects,
 * used by client processes
 */

#include "smiS.h"


/* Managed System */
typedef struct ManagedSystemS
{
    struct UTCtime systemCreationTime;
} ManagedSystemS;

#define MSYS_ATTRS      1



/* T-Layer Subsystem */
typedef struct T_LayerSubsystemS
{
    char dummy;         /* can't declare a void */
} T_LayerSubsystemS;

#define TSUBSYS_ATTRS   0



/* T-Layer Entity */
typedef struct T_LayerEntityS
{
/*
    NSAPaddrS boundNSAP;
    struct TSAPs *supportedTSAPs;
 */
    int activeEntityInvocations;
    int activeConnections;
    int previousConnections;
    int crTPDUSuccessfulIn;
    int crTPDUSuccessfulOut;
    int crTPDUUnsuccessfulIn;
    int crTPDUUnsuccessfulOut;
    int crTPDUCongestion;
    int crTPDUConfigurationError;
    int tProtocolError;
    int tPDUChecksumError;
    int numberTPDUSent;
    int numberTPDUReceived;
    int numberTPDURetransmitted;
} T_LayerEntityS;

#define TENT_ATTRS      14



/* T-Entity Invocation */
typedef struct T_EntityInvocationS
{
    struct UTCtime creationTime;
    int activeConnections;
    int crTPDUSuccessfulIn;
    int crTPDUSuccessfulOut;
    int crTPDUUnsuccessfulIn;
    int crTPDUUnsuccessfulOut;
    int crTPDUCongestion;
    int crTPDUConfigurationError;
    int tProtocolError;
    int tPDUChecksumError;
} T_EntityInvocationS;

#define EINV_ATTRS      10



/* T-Connection Endpoint */
typedef struct T_ConnectionEndpointS
{
    struct UTCtime creationTime;
    struct TSAPaddrS sourceTAddr, destTAddr;
    int numberTPDUSent;
    int numberTPDUReceived;
    int numberTPDURetransmitted;
    int numberBytesSent;
    int numberBytesReceived;
    int numberBytesRetransmitted;
    T_ProtocolState tProtocol;
} T_ConnectionEndpointS;

#define CEPT_ATTRS      10



/* Counter Threshold */
typedef struct C_ThresholdS
{
    int  compLevels[10];
    int  offsetValue;
    int  onoffSwitch;
} C_ThresholdS;

#define CTHLD_ATTRS     3



/* Gauge Threshold */
typedef struct G_ThresholdS
{
    int  compLevels[10];
    int  hysterInterval;
    int  onoffSwitch;
} G_ThresholdS;

#define GTHLD_ATTRS     3



/* Report Control */
typedef struct ReportControlS
{
    destinationS  *listofdests;
} ReportControlS;

#define RPCTL_ATTRS     1


#define M_SYSTEM        0
#define T_SUBSYS        1
#define T_ENTITY        2
#define T_EINV          3
#define T_CEPT          4
#define C_THLD          5
#define G_THLD          6
#define RPCTL           7

#define NO_EINVS        20
#define NO_CEPTS        21
#define NO_CONN         22
#define CONN_BROKEN     23


#define ISODE           100     /* single T_ENTITY instance */



typedef struct ManagedObjectS
{
    int  type;
    union {
	struct ManagedSystemS  *Msys;
	struct T_LayerSubsystemS  *Tsubsys;
	struct T_LayerEntityS  *Tent;
	struct T_EntityInvocationS  *Einv;
	struct T_ConnectionEndpointS  *Cept;
	struct C_ThresholdS  *Cthld;
	struct G_ThresholdS  *Gthld;
	struct ReportControlS  *Rpctl;
    } un;
#   define msys    un.Msys
#   define tsubsys un.Tsubsys
#   define tent    un.Tent
#   define einv    un.Einv
#   define cept    un.Cept
#   define cthld   un.Cthld
#   define gthld   un.Gthld
#   define rpctl   un.Rpctl
} ManagedObjectS;
