/*
 * Copyright (c) 1988 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/* objects.h - definitions for specific Managed Objects */


#include "smi.h"

struct X            /* MO for testing purposes */
    {
    Counter sheep;  /* one attribute, which counts sheep */
    };

/*
 * Object Identifiers: I have picked on the node 2.37 as a starting
 * point for our OIDs.
 * uclmanagement OBJECT IDENTIFIER ::= {2 37}
 * uclmo OID ::= {uclmanagement managedobjects(1)} -- node for managed
 *      -- objects
 * uclevent OID ::= {uclmanagement events(2)} -- node for event types
 */

/* Managed System */
/* {uclmo 1},  Type ID {uclmo 1 id(1)} */
typedef struct ManagedSystem
    {
    struct UTCtime systemCreationTime;          /* 2 */
    DefinedEvent systemCreationEvent;           /* 3,  {uclevent 1} */
    DefinedEvent systemShutdownEvent;           /* 4,  {uclevent 2} */
    } ManagedSystem;

/* T-Layer Subsystem */
/* {uclmo 2},  Type ID {uclmo 2 id(1)} */
typedef struct T_LayerSubsystem
    {
    char dummy; /* can't declare a void */
    } T_LayerSubsystem;

/* T-Layer Entity */
/* {uclmo 3},  Type ID {uclmo 3 name(1)} */
typedef struct T_LayerEntity
    {
    struct NSAPaddr * boundNSAP;                /* 2 */
#   define TLE_MAXTSAPS 20
    struct TSAPaddr * supportedTSAPs[TLE_MAXTSAPS]; /* 3 */
    Gauge activeEntityInvocations;              /* 4 */
    Gauge activeConnections;                    /* 5 */
    Counter previousConnections;                /* 6 */
    Counter crTPDUSuccessfulIn;                 /* 7 */
    Counter crTPDUSuccessfulOut;                /* 8 */
    Counter crTPDUUnsuccessfulIn;               /* 9 */
    Counter crTPDUUnsuccessfulOut;              /* 10 */
    Counter crTPDUCongestion;                   /* 11 */
    Counter crTPDUConfigurationError;           /* 12 */
    Counter tProtocolError;                     /* 13 */
    Counter tPDUChecksumError;                  /* 14 */
    Counter numberTPDUSent;                     /* 15 */
    Counter numberTPDUReceived;                 /* 16 */
    Counter numberTPDURetransmitted;            /* 17 */
    } T_LayerEntity;

/* T-Entity Invocation */
/* {uclmo 4},  Type ID {uclmo 4 processId(1)} */
typedef struct T_EntityInvocation
    {
    struct UTCtime creationTime;                /* 2 */
    Gauge activeConnections;                    /* 3 */
    Counter crTPDUSuccessfulIn;                 /* 4 */
    Counter crTPDUSuccessfulOut;                /* 5 */
    Counter crTPDUUnsuccessfulIn;               /* 6 */
    Counter crTPDUUnsuccessfulOut;              /* 7 */
    Counter crTPDUCongestion;                   /* 8 */
    Counter crTPDUConfigurationError;           /* 9 */
    Counter tProtocolError;                     /* 10 */
    Counter tPDUChecksumError;                  /* 11 */
    DefinedEvent crTPDUUnsuccessInThldEvent;    /* 12,  {uclevent 3} */
    DefinedEvent crTPDUUnsuccessOutThldEvent;   /* 13,  {uclevent 4} */
    DefinedEvent crTPDUCongestionThldEvent;     /* 14,  {uclevent 5} */
    DefinedEvent crTPDUConfigErrorThldEvent;    /* 15,  {uclevent 6} */
    DefinedEvent tProtocolErrorThldEvent;       /* 16,  {uclevent 7} */
    DefinedEvent tPDUChecksumErrorThldEvent;    /* 17,  {uclevent 8} */
    DefinedEvent tEntinvCreationEvent;          /* 18,  {uclevent 9} */
    DefinedEvent tEntinvShutdownEvent;          /* 19,  {uclevent 10} */
    } T_EntityInvocation;

/* T-Connection Endpoint */
/* {uclmo 5},  Type ID {uclmo 5 localConnectionEndpointId(1)} */
typedef struct T_ConnectionEndpoint
    {
    struct UTCtime creationTime;                /* 2 */
    struct TSAPaddr *sourceAddress,             /* 3 */
		    *destinationAddress;        /* 4 */
    Counter numberTPDUSent;                     /* 5 */
    Counter numberTPDUReceived;                 /* 6 */
    Counter numberTPDURetransmitted;            /* 7 */
    Counter numberBytesSent;                    /* 8 */
    Counter numberBytesReceived;                /* 9 */
    Counter numberBytesRetransmitted;           /* 10 */
    T_ProtocolState tProtocol;                  /* 11 */
    DefinedEvent numberTPDUSentThldEvent;       /* 12,  {uclevent 11} */
    DefinedEvent numberTPDUReceivedThldEvent;   /* 13,  {uclevent 12} */
    DefinedEvent numberTPDURetransThldEvent;    /* 14,  {uclevent 13} */
    DefinedEvent tCeptCreationEvent;            /* 15,  {uclevent 14} */
    DefinedEvent tCeptShutdownEvent;            /* 16,  {uclevent 15} */
    } T_ConnectionEndpoint;
