--
-- Copyright (c) 1988 University College London
-- All rights reserved.
--
-- Redistribution and use in source and binary forms are permitted
-- provided that the above copyright notice and this paragraph are
-- duplicated in all such forms and that any documentation,
-- advertising materials, and other materials related to such
-- distribution and use acknowledge that the software was developed
-- by the Department of Computer Science, University College London.
-- The name of the University may not be used to
-- endorse or promote products derived from this software without
-- specific prior written permission.
-- THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
-- IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
-- WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
--

-- reports.py - build the Event Report formats

-- By Simon Walton, October 1988
-- Modified by George Pavlou, April 1990


Reports DEFINITIONS
IMPLICIT TAGS  ::=

%{
#include <stdio.h>
#include <isode/isoaddrs.h>
#include "objects.h"

#define PEPYPARM caddr_t
%}

BEGIN

IMPORTS Counter, Gauge, TAddress, T-ProtocolState FROM MIB;

ENCODER build


-- pointed by the systemCreationEvent and systemShutdownEvent

ManagedSystemReport [[P ManagedSystem* ]] ::=  SEQUENCE
    {
    systemCreationTime  [0]  GeneralizedTime
	[[s utct2str(&parm -> systemCreationTime)]]
    }


-- pointed by all the T-EntityInvocation counter threshold events
-- through report control 2

TEinvReport [[P T_EntityInvocation* ]] ::=  SEQUENCE
    {
    creationTime                [0]  GeneralizedTime
	[[s utct2str(&parm -> creationTime)]],
    activeConnections           [1] Gauge
        [[p &parm -> activeConnections]],
    crTPDUSuccessfulIn          [2]  Counter
        [[p &parm -> crTPDUSuccessfulIn]],
    crTPDUSuccessfulOut         [3]  Counter
        [[p &parm -> crTPDUSuccessfulOut]],
    crTPDUUnsuccessfulIn        [4]  Counter
        [[p &parm -> crTPDUUnsuccessfulIn]],
    crTPDUUnsuccessfulOut       [5]  Counter
        [[p &parm -> crTPDUUnsuccessfulOut]],
    crTPDUCongestion            [6]  Counter
        [[p &parm -> crTPDUCongestion]],
    crTPDUConfigurationError    [7]  Counter
        [[p &parm -> crTPDUConfigurationError]],
    tProtocolError              [8]  Counter
        [[p &parm -> tProtocolError]],
    tPDUChecksumError           [9]  Counter
        [[p &parm -> tPDUChecksumError]]
    }


-- pointed by all the T-ConnectionEndpoint counter threshold events
-- through report control 3

-- takes a pointer to a Cept MO
TCeptReport [[P struct ManagedObject* ]]
    %{ T_ConnectionEndpoint * CeptAt; T_EntityInvocation * InvAt; %}
    ::=  SEQUENCE
    %{ CeptAt = (T_ConnectionEndpoint*) parm -> attributes;
    InvAt = (T_EntityInvocation*) parm -> parent -> attributes; %}
    {
    creationTime                [0]  GeneralizedTime
	[[s utct2str(&CeptAt -> creationTime)]],
    sourceTAddress              [1]  TAddress
        [[p CeptAt -> sourceAddress ]] OPTIONAL
	<< CeptAt -> sourceAddress >>,
    destinationTAddress         [2]  TAddress
        [[p CeptAt -> destinationAddress ]] OPTIONAL
	<< CeptAt -> destinationAddress >>,
    numberTPDUSent              [3]  Counter
        [[p &CeptAt -> numberTPDUSent ]],
    numberTPDUReceived          [4]  Counter
        [[p &CeptAt -> numberTPDUReceived ]],
    numberTPDURetransmitted     [5]  Counter
        [[p &CeptAt -> numberTPDURetransmitted ]],
    numberBytesSent             [6]  Counter
        [[p &CeptAt -> numberBytesSent ]],
    numberBytesReceived         [7]  Counter
        [[p &CeptAt -> numberBytesReceived ]],
    numberBytesRetransmitted    [8]  Counter
        [[p &CeptAt -> numberBytesRetransmitted ]],
    tProtocol                   [9] T-ProtocolState
        [[p (int*) &CeptAt -> tProtocol ]],
    activeConnections           [10] Gauge
        [[p &InvAt -> activeConnections ]],
    crTPDUSuccessfulIn          [11] Counter
        [[p &InvAt -> crTPDUSuccessfulIn ]],
    crTPDUSuccessfulOut         [12] Counter
        [[p &InvAt -> crTPDUSuccessfulOut ]]
    }


-- takes a pointer to a Cept MO
TEntityReport [[P T_LayerEntity* ]]
    ::=  SEQUENCE
    {
    activeEntityInvocations     [0]   Gauge
        [[p &parm -> activeEntityInvocations ]],
    activeConnections           [1]   Gauge
        [[p &parm -> activeConnections ]],
    previousConnections         [2]   Counter
        [[p &parm -> previousConnections ]],
    crTPDUSuccessfulIn          [3]   Counter
        [[p &parm -> crTPDUSuccessfulIn ]],
    crTPDUSuccessfulOut         [4]   Counter
        [[p &parm -> crTPDUSuccessfulOut ]],
    crTPDUUnsuccessfulIn        [5]   Counter
        [[p &parm -> crTPDUUnsuccessfulIn ]],
    crTPDUUnsuccessfulOut       [6]  Counter
        [[p &parm -> crTPDUUnsuccessfulOut ]],
    crTPDUCongestion            [7]  Counter
        [[p &parm -> crTPDUCongestion ]],
    crTPDUConfigurationError    [8]  Counter
        [[p &parm -> crTPDUConfigurationError ]],
    tProtocolError              [9]  Counter
        [[p &parm -> tProtocolError ]],
    tPDUChecksumError           [10]  Counter
        [[p &parm -> tPDUChecksumError]],
    numberTPDUSent              [11]  Counter
        [[p &parm -> numberTPDUSent ]],
    numberTPDUReceived          [12]  Counter
        [[p &parm -> numberTPDUReceived ]],
    numberTPDURetransmitted     [13]  Counter
        [[p &parm -> numberTPDURetransmitted ]]
    }

END
