--
-- Copyright (c) 1988 University College London
-- All rights reserved.
--
-- Redistribution and use in source and binary forms are permitted
-- provided that the above copyright notice and this paragraph are
-- duplicated in all such forms and that any documentation,
-- advertising materials, and other materials related to such
-- distribution and use acknowledge that the software was developed
-- by the Department of Computer Science, University College London.
-- The name of the University may not be used to
-- endorse or promote products derived from this software without
-- specific prior written permission.
-- THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
-- IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
-- WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
--

-- Pepy directives to parse the event reports

-- By George Pavlou, October 1988


Report  DEFINITIONS
IMPLICIT TAGS  ::=

%{
#include <stdio.h>
#include "objectS.h"

extern T_EntityInvocationS  *Einv;
%}


BEGIN

IMPORTS
        Counter, Gauge, TAddress, TProtocolState
                                        FROM  MIB;


DECODER parse

-- pointed by the systemCreationEvent and systemShutdownEvent
-- of ManagedSystem  (report control 1)

ManagedSystemReport  [[P ManagedSystemS **]]
    %{  ManagedSystemS **Msys = parm; char *time; %}
    ::=  SEQUENCE
    %{  if ((*Msys = (ManagedSystemS *) calloc(1, sizeof(ManagedSystemS)))
                == (ManagedSystemS *) NULL)
            return (NOTOK);
    %}
    {
    systemCreationTime		[0]  GeneralizedTime [[s time]]
        %{ (*Msys) -> systemCreationTime =
		*str2utct(time, strlen(time)); /* struct copy */ %}
    }




-- pointed by the tEinvCreationEvent, tEinvShutdownEvent
-- and all the T-EntityInvocation counter threshold events
-- (report control 2)

TEinvReport [[P T_EntityInvocationS **]]
    %{  T_EntityInvocationS **Einv = parm; char *time; %}
    ::=  SEQUENCE
    %{  if ((*Einv = (T_EntityInvocationS *)
                                calloc(1, sizeof(T_EntityInvocationS)))
                == (T_EntityInvocationS *) NULL)
            return (NOTOK);
    %}
    {
    creationTime                [0]  GeneralizedTime [[s time]]
        %{ (*Einv) -> creationTime =
		*str2utct(time, strlen(time)); /* struct copy */ %},
    activeConnections           [1]  Gauge
                                    [[p &(*Einv)->activeConnections]],
    crTPDUSuccessfulIn          [2]  Counter
                                    [[p &(*Einv)->crTPDUSuccessfulIn]],
    crTPDUSuccessfulOut         [3]  Counter
                                    [[p &(*Einv)->crTPDUSuccessfulOut]],
    crTPDUUnsuccessfulIn        [4]  Counter
                                    [[p &(*Einv)->crTPDUUnsuccessfulIn]],
    crTPDUUnsuccessfulOut       [5]  Counter
                                    [[p &(*Einv)->crTPDUUnsuccessfulOut]],
    crTPDUCongestion            [6]  Counter
                                    [[p &(*Einv)->crTPDUCongestion]],
    crTPDUConfigurationError    [7]  Counter
                                    [[p &(*Einv)->crTPDUConfigurationError]],
    tProtocolError              [8]  Counter
                                    [[p &(*Einv)->tProtocolError]],
    tPDUChecksumError           [9]  Counter
                                    [[p &(*Einv)->tPDUChecksumError]]
    }



-- pointed by the tCeptCreationEvent, tCeptShutdownEvent
-- and all the T-ConnectionEndpoint counter threshold events
-- (report control 3)

TCeptReport [[P T_ConnectionEndpointS **]]
    %{ T_ConnectionEndpointS **Cept = parm; char *time; %}
    ::=  SEQUENCE
    %{  if ((*Cept = (T_ConnectionEndpointS *)
                                calloc(1, sizeof(T_ConnectionEndpointS)))
                == (T_ConnectionEndpointS *) NULL)
            return (NOTOK);
        if (( Einv = (T_EntityInvocationS *)
                                calloc(1, sizeof(T_EntityInvocationS)))
                == (T_EntityInvocationS *) NULL)
            return (NOTOK);
    %}
    {
    creationTime                [0]  GeneralizedTime [[s time]]
        %{ (*Cept) -> creationTime =
		*str2utct(time, strlen(time)); /* struct copy */ %},
    sourceTAddress              [1]  TAddress
                                [[p &(*Cept)->sourceTAddr]] OPTIONAL
				<< &(*Cept)->sourceTAddr >>,
    destinationTAddress         [2]  TAddress
                                [[p &(*Cept)->destTAddr]] OPTIONAL
				<< &(*Cept)->destTAddr >>,
    numberTPDUSent              [3]  Counter
                                    [[p &(*Cept)->numberTPDUSent]],
    numberTPDUReceived          [4]  Counter
                                    [[p &(*Cept)->numberTPDUReceived]],
    numberTPDURetransmitted     [5]  Counter
                                    [[p &(*Cept)->numberTPDURetransmitted]],
    numberBytesSent             [6]  Counter
                                    [[p &(*Cept)->numberBytesSent]],
    numberBytesReceived         [7]  Counter
                                    [[p &(*Cept)->numberBytesReceived]],
    numberBytesRetransmitted    [8]  Counter
                                    [[p &(*Cept)->numberBytesRetransmitted]],
    tProtocol                   [9]  TProtocolState
                                    [[p &(*Cept)->tProtocol]],
--  the following attributes belong to the parent T-Einv
    activeConnections           [10] Gauge
                                    [[p &Einv->activeConnections]],
    crTPDUSuccessfulIn          [11] Counter
                                    [[p &Einv->crTPDUSuccessfulIn]],
    crTPDUSuccessfulOut         [12] Counter
                                    [[p &Einv->crTPDUSuccessfulOut]]
    }



-- pointed by the tCeptCreationEvent and tCeptShutdownEvent
-- (report control 4)

TEntityReport [[P T_LayerEntityS **]]
    %{  T_LayerEntityS **Tent = parm; %}
    ::=  SEQUENCE
    %{  if ((*Tent = (T_LayerEntityS *)
                                calloc(1, sizeof(T_LayerEntityS)))
                == (T_LayerEntityS *) NULL)
            return (NOTOK);
    %}
    {
    activeEntityInvocations     [0]   Gauge
                                    [[p &(*Tent)->activeEntityInvocations]],
    activeConnections           [1]   Gauge
                                    [[p &(*Tent)->activeConnections]],
    previousConnections         [2]   Counter
                                    [[p &(*Tent)->previousConnections]],
    crTPDUSuccessfulIn          [3]   Counter
                                    [[p &(*Tent)->crTPDUSuccessfulIn]],
    crTPDUSuccessfulOut         [4]   Counter
                                    [[p &(*Tent)->crTPDUSuccessfulOut]],
    crTPDUUnsuccessfulIn        [5]   Counter
                                    [[p &(*Tent)->crTPDUUnsuccessfulIn]],
    crTPDUUnsuccessfulOut       [6]   Counter
                                    [[p &(*Tent)->crTPDUUnsuccessfulOut]],
    crTPDUCongestion            [7]   Counter
                                    [[p &(*Tent)->crTPDUCongestion]],
    crTPDUConfigurationError    [8]   Counter
                                    [[p &(*Tent)->crTPDUConfigurationError]],
    tProtocolError              [9]   Counter
                                    [[p &(*Tent)->tProtocolError]],
    tPDUChecksumError           [10]  Counter
                                    [[p &(*Tent)->tPDUChecksumError]],
    numberTPDUSent              [11]  Counter
                                    [[p &(*Tent)->numberTPDUSent]],
    numberTPDUReceived          [12]  Counter
                                    [[p &(*Tent)->numberTPDUReceived]],
    numberTPDURetransmitted     [13]  Counter
                                    [[p &(*Tent)->numberTPDURetransmitted]]
    }


END
